@@ -227,11 +227,12 @@ def postimport_hook(repo, tag, version, options):
              extra_env=env)()
 
 
-def postunpack_hook(repo, tmp_dir, options):
+def postunpack_hook(repo, tmp_dir, sources, options):
     if options.postunpack:
         Hook('Postunpack', options.postunpack,
              extra_env={'GBP_GIT_DIR': repo.git_dir,
-                        'GBP_TMP_DIR': tmp_dir}
+                        'GBP_TMP_DIR': tmp_dir,
+                        'GBP_SOURCES_DIR': sources[0].unpacked}
              )(dir=tmp_dir)
 
 
@@ -292,12 +293,17 @@ def debian_branch_merge_by_merge(repo, tag, version, options):
     repo.set_branch(branch)
 
 
-def unpack_tarballs(name, sources, version, options):
+def unpack_tarballs(repo, name, sources, version, options):
     tmpdir = tempfile.mkdtemp(dir='../')
     if not sources[0].is_dir():  # Unpack main tarball
         sources[0].unpack(tmpdir, options.filters)
         gbp.log.debug("Unpacked '%s' to '%s'" % (sources[0].path, sources[0].unpacked))
 
+    try:
+        postunpack_hook(repo, tmpdir, sources, options)
+    except gbpc.CommandExecFailed:
+        raise GbpError()  # The hook already printed an error message
+
     if orig_needs_repack(sources[0], options):
         gbp.log.debug("Filter pristine-tar: repacking '%s' from '%s'" % (sources[0].path,
                                                                          sources[0].unpacked))
@@ -472,11 +478,7 @@ def main(argv):
         if repo.bare:
             set_bare_repo_options(options)
 
-        sources, tmpdir = unpack_tarballs(name, sources, version, options)
-        try:
-            postunpack_hook(repo, tmpdir, options)
-        except gbpc.CommandExecFailed:
-            raise GbpError()  # The hook already printed an error message
+        sources, tmpdir = unpack_tarballs(repo, name, sources, version, options)
 
         if options.verbose:
             for source in sources:
